﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device library
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Cpn.Sensor.Communication
{
  public class DevicePort : SerialPort
  {
    public DevicePort ( string portname )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      PortName = portname;
      BaudRate = 1_000_000;
      DataBits = 8;
      StopBits = StopBits.One;
      Parity = Parity.None;
      Handshake = Handshake.None;
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    public static int DefaultReadDelay => 100;


    /// <summary>
    /// Sends a message over the serial port and wait for the answer message.
    /// </summary>
    /// <param name="txmsg">Message to transmit.</param>
    /// <param name="readdelay">Delay between write and read in ms.</param>
    /// <returns>The received message</returns>
    public Message Transmit ( Message txmsg, int readdelay )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var tosend = txmsg.ByteSerialize();

      Write ( tosend.ToArray(), 0, tosend.Count );
      Thread.Sleep ( readdelay );

      var toread = new byte[BytesToRead];
      Read ( toread, 0, BytesToRead );

      var rxmsg = Message.ByteDeserialize(new List<byte>(toread));

      if ( "error" == rxmsg.Command )
      {
        var txt = "Device response failure : " + rxmsg.Parameter;
        throw new Exception ( txt );
      }

      return rxmsg;
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
