﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device library
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;
using System.Text;

namespace Cpn.Sensor.Communication
{
  public partial class Message
  {
    static uint Crc32 ( byte[] buffer, uint offset, uint count, uint previouscrc )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      if ( null == buffer ) throw new ArgumentNullException ( nameof ( buffer ) );

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      // source : http://create.stephan-brumme.com/crc32

      const uint polynomial = 0xEDB88320;

      uint crc = ~previouscrc;

      for ( uint index = offset; index < offset + count; index++ )
      {
        crc ^= buffer[index];

        for ( uint bit = 0; bit < 8; bit++ )
        {
          crc = ( crc >> 1 ) ^ ( crc & 1 ) * polynomial;
        }
      }

      return ~crc;
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
    
    
    public UInt32 Checksum
    {
      get
      {
        //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        var sx = string.Empty;
        sx += AddressHexString;
        sx += CharSOT;
        sx += Complete;
        sx += CharETX;

        var bytes = Encoding.UTF8.GetBytes(sx);
        return Crc32 ( bytes, 0, (uint)bytes.Length, 0 );
      }
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }

    public string ChecksumHexString => Checksum.ToString ( "X08" );
    public static int ChecksumHexStringLength => 8;
  }
}
