﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device library
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


namespace Cpn.Sensor.Communication
{
  public partial class Message
  {
    /// <summary>Frame control character - start of header</summary>
    public const char CharSOH = '\x01';

    /// <summary>Frame control character - start of text</summary>
    public const char CharSOT = '\x02';

    /// <summary>Frame control character - end of text</summary>
    public const char CharETX = '\x03';

    /// <summary>Frame control character - end of transmission</summary>
    public const char CharEOT = '\x04';

    /// <summary>Frame control character - command / parameter separator</summary>
    public const char CharSEP = '\x20';

    /// <summary>Frame control character - parameter list separator</summary>
    public const char CharLST = ';';


    /// <summary>Frame control character - start of header</summary>
    public const byte ByteSOH = (byte)CharSOH;

    /// <summary>Frame control character - start of text</summary>
    public const byte ByteSOT = (byte)CharSOT;

    /// <summary>Frame control character - end of text</summary>
    public const byte ByteETX = (byte)CharETX;

    /// <summary>Frame control character - end of transmission</summary>
    public const byte ByteEOT = (byte)CharEOT;

    /// <summary>Frame control character - command / parameter separator</summary>
    public const byte ByteSEP = (byte)CharSEP;

    /// <summary>Frame control character - parameter list separator</summary>
    public const byte ByteLST = (byte)CharLST;


    /// <summary>Minimum number of bytes in message</summary>
    /// <remarks>
    /// SOH + ADDR + SOT + ( DATA      ) + ETX + CHK + EOT
    /// 1   + 8    + 1   + ( 1 + 0 + 0 ) + 1   + 8   + 1
    /// </remarks>
    public const int MinimumMessageLength = 1 + 8 + 1 + (1 + 0 + 0) + 1 + 8 + 1;

    /// <summary>Minimum number of bytes in message</summary>
    /// SOH + ADDR + SOT + ( DATA         ) + ETX + CHK + EOT
    /// 1   + 8    + 1   + ( 32 + 1 + 256 ) + 1   + 8   + 1
    public const int MaximumMessageLength = 1 + 8 + 1 + (32 + 1 + 256) + 1 + 8 + 1;
  }
}
