﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device library
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.Globalization;


namespace Cpn.Sensor.Communication
{
  public partial class Message
  {
    public IReadOnlyList<string> ParameterAsStringList
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      get
      {
        if ( string.IsNullOrEmpty ( Parameter ) ) return new List<string> ();
        var items = Parameter.Split(new char[] { CharLST });
        return new List<string> ( items );
      }
      set
      {
        Parameter = string.Empty;

        for ( var index = 0; index < value.Count; index++ )
        {
          if ( 0 != index ) Parameter += CharLST;
          Parameter += value[index];
        }
      }
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    public IReadOnlyList<double> ParameterAsDoubleList
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      get
      {
        var list = new List<double>();
        foreach ( var value in ParameterAsStringList )
        {
          var ns = NumberStyles.Float;
          var pr = NumberFormatInfo.InvariantInfo;
          if ( double.TryParse ( value, ns, pr, out double result ) )
            list.Add ( result );
          else
            throw new FormatException ( $"Parameter '{Parameter}' is not a double number list" );
        }

        return list;
      }
      set
      {
        Parameter = string.Empty;

        for ( var index = 0; index < value.Count; index++ )
        {
          if ( 0 != index ) Parameter += CharLST;
          Parameter += value[index];
        }
      }
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
