﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device library
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.Text;

namespace Cpn.Sensor.Communication
{
  public partial class Message
  {
    public List<byte> ByteSerialize ()
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      // 0    1     2     3     4     5     6     7     8     9   10                      x-9 x-8  x-7  x-6  x-5  x-4  x-3  x-2  x-1  x
      // SOH  ADDR7 ADDR6 ADDR5 ADDR4 ADDR3 ADDR2 ADDR1 ADDR0 SOT command SPACE parameter ETX CHK7 CHK6 CHK5 CHK4 CHK3 CHK2 CHK1 CHK0 EOT
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var bytes     = new List<byte>();
      var encoding  = Encoding.UTF8;
      var msgbytes  = encoding.GetBytes(this.Complete);
      var addrbytes = encoding.GetBytes(AddressHexString);
      var chksbytes = encoding.GetBytes ( ChecksumHexString );

      bytes.Add ( ByteSOH );
      bytes.AddRange ( addrbytes );
      bytes.Add ( ByteSOT );
      bytes.AddRange ( msgbytes );
      bytes.Add ( ByteETX );
      bytes.AddRange ( chksbytes );
      bytes.Add ( ByteEOT );

      if ( MinimumMessageLength > bytes.Count )
        throw new FormatException ( $"To few characters ( < {MinimumMessageLength} )" );
      if ( MaximumMessageLength < bytes.Count )
        throw new FormatException ( $"To much characters ( > {MaximumMessageLength} )" );

      return bytes;
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    public static Message ByteDeserialize ( List<byte> bytes )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      // 0    1     2     3     4     5     6     7     8     9   10                      x-9 x-8  x-7  x-6  x-5  x-4  x-3  x-2  x-1  x
      // SOH  ADDR7 ADDR6 ADDR5 ADDR4 ADDR3 ADDR2 ADDR1 ADDR0 SOT command SPACE parameter ETX CHK7 CHK6 CHK5 CHK4 CHK3 CHK2 CHK1 CHK0 EOT
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var message = new Message();

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      if ( MinimumMessageLength > bytes.Count )
        throw new FormatException ( $"To few characters ( < {MinimumMessageLength} )" );
      if ( MaximumMessageLength < bytes.Count )
        throw new FormatException ( $"To much characters ( > {MaximumMessageLength} )" );

      var possoh = bytes.IndexOf(ByteSOH);
      var possot = bytes.IndexOf(ByteSOT);
      var posetx = bytes.IndexOf(ByteETX);
      var poseot = bytes.IndexOf(ByteEOT);

      if ( -1 == possoh ) throw new FormatException ( $"SOH character missing" );
      if ( -1 == possot ) throw new FormatException ( $"SOT character missing" );
      if ( -1 == posetx ) throw new FormatException ( $"ETX character missing" );
      if ( -1 == poseot ) throw new FormatException ( $"EOT character missing" );

      var expectedsoh = 0;
      var expectedsot = AddressHexString.Length + 1;
      var expectedetx = bytes.Count - 1 - ChecksumHexStringLength - 1;
      var expectedeot = bytes.Count - 1;

      if ( expectedsoh != possoh ) throw new FormatException ( $"SOH character position invalid" );
      if ( expectedsot != possot ) throw new FormatException ( $"SOT character position invalid" );
      if ( expectedetx != posetx ) throw new FormatException ( $"ETX character position invalid" );
      if ( expectedeot != poseot ) throw new FormatException ( $"EOT character position invalid" );

      var msgbybeg = 1 + AddressHexString.Length + 1;
      var msgbylen = bytes.Count - msgbybeg - 1 - ChecksumHexStringLength - 1;
      var msgbytes = bytes.GetRange(msgbybeg, msgbylen);
      var chkbytes = bytes.GetRange(bytes.Count - ChecksumHexStringLength - 1, ChecksumHexStringLength);

      message.Complete = Encoding.UTF8.GetString ( msgbytes.ToArray () );

      if ( string.IsNullOrEmpty ( message.Command ) ) throw new Exception ( $"Command must not be empty" );

      var chkcalc = message.ChecksumHexString.ToUpper();
      var chkread = Encoding.ASCII.GetString(chkbytes.ToArray()).ToUpper();

      if ( chkread != chkcalc ) throw new FormatException ( $"Checksum failed ( read {chkread} - calc {chkcalc} )" );
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      return message;
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
