﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device library
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System.ComponentModel;
using System.Linq;


namespace Cpn.Sensor
{
  public partial class Device : Component
  {
    /// <summary>
    /// Starts module measurement.
    /// </summary>
    public void PidStart ()
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Transmit ( "pid.start" );
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    /// <summary>
    /// Stops module measurement.
    /// </summary>
    public void PidStop ()
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Transmit ( "pid.stop" );
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    /// <summary>
    /// Read pid measurement values.
    /// </summary>
    /// <returns>Float array of measurement values.</returns>
    public double[] PidValues ()
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var rxmsg = Transmit("pid.values ?");
      return rxmsg.ParameterAsDoubleList.ToArray();
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    /// <summary>Read pid state.</summary>
    /// <returns>State as text.</returns>
    public string PidState ()
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      return Transmit("pid.state ?").Parameter;
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    /// <summary>Read pid error.</summary>
    /// <returns>State as text.</returns>
    public string PidError ()
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      return Transmit ( "pid.error ?" ).Parameter;
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
