﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device sample console application
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;


namespace Cpn.Sensor
{
  class Program
  {
    static void Main ( string[] args )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var portname = "COM1";
      if ( 1 == args.Length ) portname = args[0];

      Console.WriteLine ( "Read calibration data on port " + portname );
      Console.WriteLine ();

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      try
      {
        if ( !Device.Identify ( portname ) )
          throw new Exception ( "Device identification FAILED on " + portname );

        using ( var device = new Device ( portname ) )
        {
          device.Connect ();

          var values = device.Transmit("pid.calib ?").ParameterAsDoubleList;
          var calib = CalibValues.FromMessageValues(values);

          Console.WriteLine ( "Module calibration values :" );
          Console.WriteLine ();

          calib.WriteValuesToConsole ();

          Console.WriteLine ();
          Console.WriteLine ( "Press any key to exit" );
          Console.ReadKey ();
        }
      }
      catch ( Exception exc )
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine ();
        Console.WriteLine ( "ERROR : " );
        Console.WriteLine ( exc.Message );
        Console.WriteLine ();
        Console.ForegroundColor = ConsoleColor.White;
        Console.WriteLine ( "Press any key to exit" );
        Console.ReadKey ();
      }
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
