﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device sample console application
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;


namespace Cpn.Sensor
{
  class Program
  {
    static void Main ( string[] args )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var portname = "COM1";
      if ( 1 == args.Length ) portname = args[0];

      Console.WriteLine ( "Read measurement data on port " + portname );
      Console.WriteLine ( "  Press any key to abort" );
      Console.WriteLine ();

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      try
      {
        if ( !Device.Identify ( portname ) )
          throw new Exception ( "Device identification FAILED on " + portname );

        using ( var device = new Device ( portname ) )
        {
          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          device.Connect ();

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          device.PidStart();

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
          while ( !Console.KeyAvailable )
          {
            //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            {
              var state = device.PidState();
              var error = device.PidError();
              Console.Write ( "State 0x{0}, Error 0x{1},  ", state, error );
            }

            //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
            {
              var values = device.PidValues();

              Console.Write ( "Conc {0,8:F2} ppm,  ", values[0] );
              Console.Write ( "Curr {0,8:F2} pA,  ", values[1] );
              Console.Write ( "Temp {0,8:F1} °C,  ", values[2] );
              Console.Write ( "Humid {0,8:F1} %rH,  ", values[3] );
              Console.Write ( "Flow {0,8:F1} %", values[4] );
            }

            Console.Write ( "\r" );
          }

          device.PidStop ();

          //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
        }
      }
      catch ( Exception exc )
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine ();
        Console.WriteLine ( "ERROR : " );
        Console.WriteLine ( exc.Message );
        Console.WriteLine ();
        Console.ForegroundColor = ConsoleColor.White;
        Console.WriteLine ( "Press any key to exit" );
        Console.ReadKey ();
      }
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
