﻿//-----------------------------------------------------------------------------
// ACI PIM CPN Sensor sample program
// Cpn sensor device sample console application
//-----------------------------------------------------------------------------
// 2020 Analytical Control Instruments GmbH
//-----------------------------------------------------------------------------


using System;

namespace Cpn.Sensor
{
  class Program
  {
    static void Main ( string[] args )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      var portname = "COM1";
      if ( 1 == args.Length ) portname = args[0];

      Console.WriteLine ( "Read information on port " + portname );
      Console.WriteLine ();

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      try
      {
        if ( !Device.Identify ( portname ) )
          throw new Exception ( "Device identification FAILED on " + portname );

        using ( var device = new Device ( portname ) )
        {
          device.Connect ();

          ReadShowDeviceInfo ( device );
          Console.WriteLine (); Console.WriteLine ();

          ReadShowMeasConfig ( device );
          Console.WriteLine (); Console.WriteLine ();

          ReadShowLampInfo ( device );

          Console.WriteLine ();
          Console.WriteLine ( "Press any key to exit" );
          Console.ReadKey ();
        }
      }
      catch ( Exception exc )
      {
        Console.ForegroundColor = ConsoleColor.Red;
        Console.WriteLine ();
        Console.WriteLine ( "ERROR : " );
        Console.WriteLine ( exc.Message );
        Console.WriteLine ();
        Console.ForegroundColor = ConsoleColor.White;
        Console.WriteLine ( "Press any key to exit" );
        Console.ReadKey ();
      }
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    static void ReadShowDeviceInfo ( Device device )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Console.WriteLine ( "Device information values :" );
      Console.WriteLine ();

      var devicename = device.Transmit("device ?").Parameter;
      Console.WriteLine ( "  Name               = {0}", devicename );

      var deviceserial = device.Transmit("device.serialno ?").Parameter;
      Console.WriteLine ( "  Serialnumber       = {0}", deviceserial );

      var devicesoftware = device.Transmit("device.software ?").Parameter;
      Console.WriteLine ( "  SW Version         = {0}", devicesoftware );

      var devicehardware = device.Transmit("device.hardware ?").Parameter;
      Console.WriteLine ( "  HW Version         = {0}", devicehardware );
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }


    static void ReadShowMeasConfig ( Device device )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Console.WriteLine ( "Device Measurement config :" );
      Console.WriteLine ();

      var measconfig = device.Transmit("pid.measconfig ?").ParameterAsStringList;
      var calibmethod = measconfig[0];
      var gasident = measconfig[1];
      var gasfactor = measconfig[2];

      Console.WriteLine ( "  Calibration method = {0}", calibmethod );
      Console.WriteLine ( "  Gas identification = {0}", gasident );
      Console.WriteLine ( "  Gas result factor  = {0}", gasfactor );

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }

    static void ReadShowLampInfo ( Device device )
    {
      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      Console.WriteLine ( "Device lamp information :" );
      Console.WriteLine ();

      var lampinfo = device.Transmit("pid.lampinfo ?").ParameterAsStringList;
      var lampserial = lampinfo[0];
      var lampvariant = lampinfo[1];
      var lampontime = lampinfo[2];

      Console.WriteLine ( "  Lamp serial number = {0}", lampserial );
      Console.WriteLine ( "  Lamp variant text  = {0}", lampvariant );
      Console.WriteLine ( "  Lamp ontime        = {0} hrs", lampontime );

      //- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    }
  }
}
